package org.gcube.vremanagement;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.stream.Collectors;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class XmlManager implements VreRemPubInterface {
	private String filePath;
	private HashMap<String, Gateway> gatewaysMap;

	public XmlManager(String filePath) {
		this.filePath = filePath;
		this.gatewaysMap = new HashMap<>();
	}

	@Override
	public void parse() {
		try {
			File xmlFile = new File(filePath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(xmlFile);
			doc.getDocumentElement().normalize();
			NodeList gatewayNodes = doc.getElementsByTagName("gateways");
			
			NodeList gatewayList = gatewayNodes.item(0).getChildNodes();

			for (int i = 0; i < gatewayList.getLength(); i++) {
				Node gatewayNode = gatewayList.item(i);
				if (gatewayNode.getNodeType() == Node.ELEMENT_NODE) {
					Element gatewayElement = (Element) gatewayNode;
					String gatewayName = gatewayElement.getElementsByTagName("name").item(0).getTextContent().trim();
					String gatewayDescription = gatewayElement.getElementsByTagName("description").item(0)
							.getTextContent().trim();

					//System.out.println("gateway name:"+gatewayName);
					//System.out.println("Gateway descr:"+gatewayDescription);
					Gateway gateway = new Gateway(gatewayName, gatewayDescription);

					// Find the <vos> element within the gateway entry
					NodeList vosList = gatewayElement.getElementsByTagName("vos");
					for (int j = 0; j < vosList.getLength(); j++) {
						Node vosNode = vosList.item(j);
						if (vosNode.getNodeType() == Node.ELEMENT_NODE) {
							Element vosElement = (Element) vosNode;
							
							
							
							
							String voKey = vosElement.getElementsByTagName("key").item(0).getTextContent().trim();
							String voName = vosElement.getElementsByTagName("name").item(0).getTextContent().trim();
							String voScope = vosElement.getElementsByTagName("scope").item(0).getTextContent().trim();
												
							//getting vo
							Vos vos = new Vos(voKey,voScope, voName);
							
							//vos.printInfo();
							
							// Iterate on the vres..
							//NodeList vreList = vosElement.getElementsByTagName("vres");
							//
							//for (int k = 0; k < vreList.getLength(); k++) {
							//	Node voNode = voList.item(k);
						//		if (voNode.getNodeType() == Node.ELEMENT_NODE) {
							//		Element voElement = (Element) voNode;
									
									
							//		String voName = voElement.getElementsByTagName("name").item(0).getTextContent().trim();
							//		String voScope = voElement.getElementsByTagName("scope").item(0).getTextContent().trim();
							//		String voKey = voElement.getElementsByTagName("key").item(0).getTextContent().trim();

									// Create a new Vos object and add it to the gateway's vosList
							//		Vos vos = new Vos(voKey, voScope, voName);

									// Find the <vres> element within the VO entry
									NodeList vresList = vosElement.getElementsByTagName("vres");
									for (int l = 0; l < vresList.getLength(); l++) {
										Node vresNode = vresList.item(l);
										if (vresNode.getNodeType() == Node.ELEMENT_NODE) {
											Element vresElement = (Element) vresNode;
											// Iterate through <entry> elements representing VREs within <vres>
											NodeList vreEntryList = vresElement.getElementsByTagName("entry");
											for (int m = 0; m < vreEntryList.getLength(); m++) {
												Node vreEntryNode = vreEntryList.item(m);
												if (vreEntryNode.getNodeType() == Node.ELEMENT_NODE) {
													Element vreEntryElement = (Element) vreEntryNode;
													String vreKey = vreEntryElement.getElementsByTagName("key").item(0)
															.getTextContent().trim();
													String vreName = vreEntryElement.getElementsByTagName("name")
															.item(0).getTextContent().trim();
													String vreDescription = vreEntryElement
															.getElementsByTagName("description").item(0)
															.getTextContent().trim();
													String vreManager = vreEntryElement.getElementsByTagName("managers")
															.item(0).getTextContent().trim();
													String vreStartDate = vreEntryElement
															.getElementsByTagName("startdate").item(0).getTextContent().trim();
													String vreEndDate = vreEntryElement.getElementsByTagName("enddate")
															.item(0).getTextContent().trim();

													// Create a new Vres object and add it to the Vos's vreList
													Vres vres = new Vres(vreKey, vreName, vreDescription, vreManager,
															vreStartDate, vreEndDate);
													vos.addVres(vres);
													
													//vres.printInfo();
												}
											}
										}
									}

									// Add the Vos object to the gateway's vosList
									gateway.addVos(vos);
								//}
							//}
						}
					}

					// Store the gateway object in the gateways map
					
					gatewaysMap.put(gatewayName, gateway);
					//computeIfAbsent(gatewayName, k -> new ArrayList<>()).add(gateway);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	public void getGatewaysInfo() {
		System.out.println("Name of the Gateway:");
		gatewaysMap.keySet().stream().filter(gatewayName -> gatewayName.endsWith("Gateway"))
				.forEach(System.out::println);
	}
	// Other methods...

	@Override
	public void update() {
		// Implement update logic if needed
	}

	@Override
	public void find() {
		// Implement find logic if needed
	}

	@Override
	public void read() {
		// Implement read logic if needed
	}

	@Override
	public void write() {
		// Implement write logic if needed
	}

	//public HashMap<String, ArrayList<Gateway>> parseToHashMap() {
//		return gatewaysMap;
//	}
	
	
	public void getVosInfo(String _gateway) {
		System.out.println("VOs Information for Gateway: " + _gateway);
		
		if (gatewaysMap.containsKey(_gateway)) {
			Gateway gatewayItem = gatewaysMap.get(_gateway);
			//Gateway gateway = gatewayList.get(0); // Get the first gateway with the specified name
			ArrayList<Vos> vosList = gatewayItem.getVosList();
			if (vosList != null) {
				for (Vos vos : vosList) {
				    vos.printInfo();
					//System.out.println("Name of VREs: ");
					//ArrayList<Vres> vresList = vos.getVresList();
					//if (vresList != null && !vresList.isEmpty()) {
					//	for (Vres vre : vresList) {
					//		vre.printInfo();
					//	}
					//} else {
				//		System.out.println("No VREs found for this VOS.");
				//	}
				//	System.out.println("----------------------------------------");
				}
			} else {
				//System.out.println("No VOS found for this gateway.");
			}
		} else {
			System.out.println("No gateways found with the name: " + _gateway);
		}
	}

	// Method to find a VO within a gateway
	public void findVo(String _gateway, String _vo) {

		if (gatewaysMap.containsKey(_gateway)) 
		{
			Gateway gatewayItem = gatewaysMap.get(_gateway);
			boolean found = false;
		    ArrayList<Vos> vosList = gatewayItem.getVosList();
			if (vosList != null) {
					for (Vos vos : vosList) {
						if (vos.getName().equals(_vo)) {
							found = true;
							System.out.println("VO info for " + _vo + " in " + _gateway + ":");
							System.out.println("Name of VOS: " + vos.getName());
						}
				}
				
			}
			if (!found) {
				System.out.println("VO " + _vo + " not found in the gateway " + _gateway + ".");
			}
		} else {
			System.out.println("Gateway " + _gateway + " not found.");
		}
	}

	public void findVre(String _gateway, String _vo, String _vre) {
	    //System.out.println("VRE Information for VO: " + _vo + " in Gateway:\n " + _gateway);
	    boolean vreFound = false; // Flag to track if VRE exists
	    //ArrayList<Gateway> gatewayList = gatewaysMap.get(_gateway);
	    
	    if (gatewaysMap.containsKey(_gateway)) {
	    	
	    	Gateway gatewayItem = gatewaysMap.get(_gateway);
	            ArrayList<Vos> vosList = gatewayItem.getVosList();
	            if (vosList != null) {
	                for (Vos vos : vosList) {
	                    if (vos.getName().equals(_vo)) {
	                        ArrayList<Vres> vresList = vos.getVresList();
	                        if (vresList != null) {
	                            for (Vres vre : vresList) {
	                                if (vre.getName().equals(_vre)) {
	                                    // Print VRE information
	                                    vre.printInfo();
	                                    vreFound = true; // Set flag to true
	                                    // return; // Exit method after printing information
	                                }
	                            }
	                        }
	                    }
	                }
	         }
	    }
	    // If execution reaches here and VRE not found, print message
	    if (!vreFound) {
	        System.out.println("VRE does not exist in the " + _vo + " of the " + _gateway);
	    }
	}

	
// Other methods...
}
